<#
.SYNOPSIS
    Tests network connectivity to ControlUp Cloud URLs.
.PARAMETERS
    CUProduct   :The ControlUp product whos network connectvity need to be tested.
    edgetenant  :The name of your EdgeDX tenant (If your URL is testedge.sip.controlup.com, then "testedge" is the name of the tenant).
    location    :The location of your ControlUp RealtimeDX tenant (EU or NonEU).
    exportpath  :The file path where you want to export the results.
.PREREQS
    To test RealtimeDX Console URLs, run these tests from the machines running the RealTime Console.
    To test RealtimeDX Monitor URLs, run these tests from the machines running the ControlUp Monitor service. 
    To test Scoutbees run these tests from a Custom hive.
    To test EdgeDX run these tests from a Web Browser that will be used to manage the environment.
    To test Citrix Cloud run these tests from the ControlUp Data Collector assigned to monitor EUC Citrix environment.
    To test Microsoft Azure run these tests from the ControlUp Data Collector assigned to monitor Microsoft Azure environment.
.EXAMPLE
    Test RealtimeDX URLs:                                     Network_Check_V5.ps1 -CUProduct RealtimeDXMonitor
    Test EdgeDX URLs:                                         Network_Check_V5.ps1 -CUProduct EdgeDX -edgetenant MyOrgName
    Test all product URLs:                                    Network_Check_V5.ps1 -CUProduct All -edgetenant MyOrgName
    Test all product URLs for EU customers:                   Network_Check_V5.ps1 -CUProduct All -edgetenant MyOrgName -location EU
    Test RealtimeDX product URLs for Non EU customers:        Network_Check_V5.ps1 -CUProduct RealtimeDXMonitor -location NonEU
    Test RealtimeDX product URLs applicable to the Console:   Network_Check_V5.ps1 -CUProduct RealtimeDXConsole
.VERSION HISTORY
    V3 :Added EdgeDX check.
    V4 :Added ability to check EU or NonEU locations seperately.
    V5 :Changed user detection to use SID, added check user parameter.
    V6 :Added ability to test URLs for RealtimeDX Console only, and also changed the parameters so we now have RealtimeDXMonitor and RealtimeDXConsole
	V8 :removed rt.controlup.com check 
#>

[CmdLetBinding()]
Param (
    [Parameter(Mandatory=$false,HelpMessage='Which product do you want to test?')] [ValidateSet("RealtimeDXMonitor","RealtimeDXConsole","Scoutbees","EdgeDX","Citrix Cloud","Microsoft Azure","All")] [string]$CUProduct,
    [Parameter(Mandatory=$false,HelpMessage='EdgeDX tenant name')] [string]$edgetenant,
    [Parameter(Mandatory=$false,HelpMessage='Location')] [ValidateSet("EU","NonEU")] [string]$location,
    [Parameter(Mandatory=$false,HelpMessage='Export Path')] [String]$exportpath = $env:temp,
    [Parameter(Mandatory=$false,HelpMessage='Get ControlUp Monitor Service user')] [ValidateSet("True","False")] [String]$checksvcuser = "False"
)

$URLs_RealtimeDX=@{
'Global-Solve:' = 'mp.controlup.com'
'Global-Insights:' = 'insights-hec.controlup.com'
'Global-LoadBalancer:' = 'rt-app.controlup.com'
'NonEU-Proxy:' = 'fe2.controlup.com','fe4.controlup.com'
'NonEU-AWS:' = 's3.amazonaws.com'
'NonEU-CentralizedAuditing:' = 'cu-ca-us.controlup.com'
'EU-Proxy:' = 'fe1.controlup.com','fe3.controlup.com'
'EU-AWS:' = 's3.eu-central-1.amazonaws.com'
'EU-CentralizedAuditing:' = 'cu-ca-eu.controlup.com'
'EU-GDPRFrontEnd:' = 'uploader-eu-central-1.controlup.com'
'NonEU-ConfigurationService:' = 'rt-app-us.controlup.com'
'EU-ConfigurationService:' = 'rt-app-eu.controlup.com'
'EU-CentralConfigurationService:' = 'rt-app-eu-central-1.controlup.com'
}

$ConsoleURLs_RealtimeDX=@{
'Global-LoadBalancer:' = 'rt-app.controlup.com'
'NonEU-Proxy:' = 'fe2.controlup.com','fe4.controlup.com'
'NonEU-CentralizedAuditing:' = 'cu-ca-us.controlup.com'
'EU-Proxy:' = 'fe1.controlup.com','fe3.controlup.com'
'EU-CentralizedAuditing:' = 'cu-ca-eu.controlup.com'
'EU-GDPRFrontEnd:' = 'uploader-eu-central-1.controlup.com'
'NonEU-ConfigurationService:' = 'rt-app-us.controlup.com'
'EU-ConfigurationService:' = 'rt-app-eu.controlup.com'
'EU-CentralConfigurationService:' = 'rt-app-eu-central-1.controlup.com'
}

$URLs_Scoutbees=@{
'NonEU-Scoutbees WebSocket over HTTPS' = 'ws.scoutbees.io'
'EU-Scoutbees WebSocket over HTTPS' = 'ws.eu.scoutbees.io'
'NonEU-Scoutbees Data Collector' = 'api.scoutbees.io'
'EU-Scoutbees Data Collector' = 'api.eu.scoutbees.io'
}

$URLs_EdgeDX=@{
'Global-EdgeDX Download Manager' = 'downloads.sip.controlup.com'
'Global-EdgeDX tenant' = "$edgetenant.sip.controlup.com"
'Global-Google Maps' = 'maps.google.com'
}

$URLs_CitrixCloud=@{
'NonEU-Citrix Cloud oData API endpoint' = "api-us.cloud.com"
'EU-Citrix Cloud oData API endpoint' = "api-eu.cloud.com"
'Global-Citrix Cloud oData API endpoint' = "api-ap-s.cloud.com"
}

$URLs_MicrosoftAzure=@{
'Global-Azure Management Portal' = "management.azure.com"
}

Function Get-SidFromName ($user) {
    (New-Object System.Security.Principal.NTAccount($user)).Translate([System.Security.Principal.SecurityIdentifier]).Value
}

Function NetworkCheck {
    param (
        [Parameter(Mandatory=$true,HelpMessage='Product to test')] [String]$CUProduct,
        [Parameter(Mandatory=$true,HelpMessage='URLS to test')] $URLs
    )

    If ($CUProduct -eq "RealtimeDXMonitor"){
        $usercontext = whoami
        Write-Host "This script running as user ""$($usercontext)""" -ForegroundColor black -BackgroundColor white
        $servicecontext = (Get-CimInstance -query "select * from win32_service where name='cumonitor'")
        If ($servicecontext){
            Write-Host "ControlUp Monitor Service running as user ""$($servicecontext.startname)""" -ForegroundColor black -BackgroundColor white
                If ((Get-SidFromName $usercontext) -ne (Get-SidFromName $servicecontext.startname)){
                    Write-Host "WARNING: Script should be run as same user which is running the ControlUp Monitor Service." -ForegroundColor red -BackgroundColor white
                    Write-Host "Use the following command to open powershell as the same user that is running the ControlUp Monitor Service: `nc:\temp\psexec.exe -i -u ""$($servicecontext.startname)"" powershell.exe" -ForegroundColor red -BackgroundColor white
                }
        }
        Else {
            Write-Host "WARNING: Could not find ControlUp Monitor Service.  If you are wanting to test ControlUp Monitor connectivity please run this on a Monitor machine." -ForegroundColor red -BackgroundColor White
        }
    }

    
    [array]$results=@{}

    $URLs.GetEnumerator() | Sort-Object -property Name | ForEach-Object {
        If ($_.key -like "$location*" -OR $_.key -like "Global*"){
            Write-Host "Testing $($_.key)"
            $key = $_.key
            ForEach($dnsname in $_.value){
                Try {
                    $ips = (Resolve-DnsName -Name $dnsname | Where-Object {$_.Type -ne "AAAA"} -ErrorAction Stop).ipaddress
                    Write-Host "$dnsname resolves to the following IP address/es: $ips"
                }
                Catch {
                    $object = New-Object -TypeName PSObject
                    $object | Add-Member -Name 'Resource Name' -MemberType Noteproperty -Value $key
                    $object | Add-Member -Name 'DNS Name' -MemberType Noteproperty -Value $Error[1]
                    $object | Add-Member -Name 'Remote Address' -MemberType Noteproperty -Value "ISSUE FOUND"
                    $object | Add-Member -Name 'Success' -MemberType Noteproperty -Value "False"            
                    $results += $object
                    break
                }
                
                ForEach($ip in $ips){
                    $test = Test-NetConnection $ip -port 443
                    $object = New-Object -TypeName PSObject
                    $object | Add-Member -Name 'Resource Name' -MemberType Noteproperty -Value $key
                    $object | Add-Member -Name 'DNS Name' -MemberType Noteproperty -Value $dnsname
                    $object | Add-Member -Name 'Remote Address' -MemberType Noteproperty -Value $test.RemoteAddress
                    $object | Add-Member -Name 'Success' -MemberType Noteproperty -Value $test.TcpTestSucceeded
                    $results += $object
                }
            }
        }
    }
    $results | Format-Table -AutoSize
    Write-Host "Exporting detail to $exportpath\ControlUp_NetCheck_$CUProduct.xml"
    $results | Export-Clixml -Path "$exportpath\ControlUp_NetCheck_$CUProduct.xml" -Force
}


Write-Host "ControlUp: Network Check Script" -ForegroundColor black -BackgroundColor white
If ($checksvcuser -eq "True"){
    $servicecontext = (Get-CimInstance -query "select * from win32_service where name='cumonitor'")
    If ($servicecontext){
        Write-Host "ControlUp Monitor service is running as $($servicecontext.startname)" -ForegroundColor red -BackgroundColor white
        Write-Host "Use the following command to open powershell as the same user that is running the ControlUp Monitor Service: `nc:\temp\psexec.exe -i -u ""$($servicecontext.startname)"" powershell.exe" -ForegroundColor red -BackgroundColor white
        Exit
    }
    else {
        Write-Host "WARNING: Could not find ControlUp Monitor Service.  If you are wanting to test ControlUp Monitor connectivity please run this on a Monitor machine." -ForegroundColor red -BackgroundColor White
        Exit
    }
}

If  (!$CUProduct){
    Write-Host "You need to enter a parameter for CUProduct."
    Exit
}
If ($location) {Write-Host "$location location specified. Testing URLs for $location tenants" -ForegroundColor black -BackgroundColor white} 
Else {Write-Host "Location NOT specified. Testing all region URLs" -ForegroundColor black -BackgroundColor white}

Switch ($CUProduct){
    "RealtimeDXMonitor" {NetworkCheck $CUProduct $URLs_RealtimeDX}
    "RealtimeDXConsole" {NetworkCheck $CUProduct $ConsoleURLs_RealtimeDX}
    "Scoutbees" {NetworkCheck $CUProduct $URLs_Scoutbees}
    "EdgeDX" {If($edgetenant){NetworkCheck $CUProduct $URLs_EdgeDX}Else{Write-Output "You need to supply your Edge Tenant name in the arguments."}}
    "Citrix Cloud" {NetworkCheck $CUProduct $URLs_CitrixCloud}
    "Microsoft Azure" {NetworkCheck $CUProduct $URLs_MicrosoftAzure}
    "All" {If (!$edgetenant){Write-Host "You need to supply your Edge Tenant name in the arguments."}Else{;NetworkCheck "RealtimeDX" $URLs_RealtimeDX;NetworkCheck "Scoutbees" $URLs_Scoutbees;NetworkCheck "EdgeDX" $URLs_EdgeDX;NetworkCheck "Citrix Cloud" $URLs_CitrixCloud;NetworkCheck "Microsoft Azure" $URLs_MicrosoftAzure}}
}